#ifndef _MASSWARE_H_
#define _MASSWARE_H_

#ifndef _MODULE_
int8_t massware_init();
#endif

//! Next module Message Structure, connect this module with the next one
typedef struct PostDestMsg {
  sos_pid_t dst_mod_id;
  uint8_t msg_type;
} PACK_STRUCT PostDestMsg;

typedef struct MassOutputInterf {
	uint8_t out_msg_type;         // define the output message type
    sos_pid_t dst_mod_id;         // define the subscribed component id
    uint8_t dst_msg_type;         // define the message type of the input interface of the subscribed component
	// bool selected;                // check if this output is selected or not.
	struct MassOutputInterf *next;   // point to the next interface
} PACK_STRUCT MassOutputInterf;

//! The next four structures for synchornization
typedef struct ParamSetting {
	sos_pid_t component_pid;
	uint8_t component_msg_type;
	uint32_t value;
} PACK_STRUCT ParamSetting;

typedef struct ArchSetting {
	sos_pid_t sender_pid;
	uint8_t sender_msg_type;
	sos_pid_t recver_pid;
	uint8_t recver_msg_type;
} PACK_STRUCT ArchSetting;

typedef struct ActuatorMeta {
	uint8_t act_index;
	uint8_t param_num;
	ParamSetting * param_list;
	uint8_t conn_num;
	ArchSetting * conn_list;
} PACK_STRUCT ActuatorMeta;

typedef struct SyncRequest {
	uint8_t act_num;
	ActuatorMeta * act_list;
} PACK_STRUCT SyncRequest;

enum {
	MASS_MSG_DATA_OUTPUT            = (MOD_MSG_START + 0),
	MASS_MSG_DATA_INPUT             = (MOD_MSG_START + 1),
	MASS_MSG_SYNC_REQUEST           = (MOD_MSG_START + 2),
	MASS_MSG_SYNC_REPLY             = (MOD_MSG_START + 3),
	MASS_MSG_SYNC_REASK             = (MOD_MSG_START + 4),
	MASS_MSG_SET_DESTINATION        = (MOD_MSG_START + 5),
};

enum {
	/***
	 * we do not change the system pid because we do not want to touch sos,
	 * however, we must pay attention that the masslets or masstools can not use the same mod id with
	 * massware. for more information about the system pid, ref: $SOSROOT/modules/include/mod_pid.h
	 */
	MASSWARE_AGENT_PID        = (APP_MOD_MIN_PID + 55),
};

MassOutputInterf * search_interface(MassOutputInterf * first, uint8_t msg)
{
	if (first == NULL) return NULL;
	MassOutputInterf *temp = first;

	do {
		if (temp->out_msg_type == msg) return temp; 
		else temp = temp->next;
	} while (temp != NULL);
	return NULL;
}

void release_interface(MassOutputInterf * first)
{
	MassOutputInterf *temp;
	while (first != NULL) {
		temp = first;
		first = temp->next;
		sys_free(temp);
	}
}
#endif

